var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import { resolveMaybeThunk } from './utils/misc';
import { isObject } from './utils/is';
import { GraphQLInputObjectType, GraphQLNonNull, getNamedType } from 'graphql';

var InputTypeComposer = function () {
  function InputTypeComposer(gqType) {
    _classCallCheck(this, InputTypeComposer);

    if (!(gqType instanceof GraphQLInputObjectType)) {
      throw new Error('InputTypeComposer accept only GraphQLInputObjectType in constructor');
    }
    this.gqType = gqType;
  }

  /**
   * Get fields from a GraphQL type
   * WARNING: this method read an internal GraphQL instance variable.
   */


  _createClass(InputTypeComposer, [{
    key: 'getFields',
    value: function getFields() {
      var fields = this.gqType._typeConfig.fields;

      var fieldMap = resolveMaybeThunk(fields);

      if (isObject(fieldMap)) {
        return Object.assign({}, fieldMap);
      }
      return {};
    }
  }, {
    key: 'getFieldNames',
    value: function getFieldNames() {
      return Object.keys(this.getFields());
    }
  }, {
    key: 'hasField',
    value: function hasField(fieldName) {
      var fields = this.getFields();
      return !!fields[fieldName];
    }

    /**
     * Completely replace all fields in GraphQL type
     * WARNING: this method rewrite an internal GraphQL instance variable.
     */

  }, {
    key: 'setFields',
    value: function setFields(fields) {
      this.gqType._typeConfig.fields = function () {
        return fields;
      };
      delete this.gqType._fields; // if schema was builded, delete defineFieldMap
    }

    /**
     * Add field to a GraphQL type
     */

  }, {
    key: 'addField',
    value: function addField(fieldName, fieldConfig) {
      this.addFields(_defineProperty({}, fieldName, fieldConfig));
    }

    /**
     * Add new fields or replace existed in a GraphQL type
     */

  }, {
    key: 'addFields',
    value: function addFields(newFields) {
      this.setFields(Object.assign({}, this.getFields(), newFields));
    }

    /**
     * Get fieldConfig by name
     */

  }, {
    key: 'getField',
    value: function getField(fieldName) {
      var fields = this.getFields();

      if (fields[fieldName]) {
        return fields[fieldName];
      }

      return undefined;
    }
  }, {
    key: 'removeField',
    value: function removeField(fieldNameOrArray) {
      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      var fields = this.getFields();
      fieldNames.forEach(function (fieldName) {
        return delete fields[fieldName];
      });
      this.setFields(fields);
    }
  }, {
    key: 'clone',
    value: function clone(newTypeName) {
      if (!newTypeName) {
        throw new Error('You should provide new type name for clone() method');
      }

      var fields = this.getFields();
      var newFields = {};
      Object.keys(fields).forEach(function (fieldName) {
        newFields[fieldName] = Object.assign({}, fields[fieldName]);
      });

      return new InputTypeComposer(new GraphQLInputObjectType({
        name: newTypeName,
        fields: newFields
      }));
    }

    /**
     * Get fieldType by name
     */

  }, {
    key: 'getFieldType',
    value: function getFieldType(fieldName) {
      var field = this.getField(fieldName);
      if (field) {
        return field.type;
      }

      return undefined;
    }
  }, {
    key: 'getType',
    value: function getType() {
      return this.gqType;
    }
  }, {
    key: 'getTypeName',
    value: function getTypeName() {
      return this.gqType.name;
    }
  }, {
    key: 'setTypeName',
    value: function setTypeName(name) {
      this.gqType.name = name;
    }
  }, {
    key: 'getDescription',
    value: function getDescription() {
      return this.gqType.description || '';
    }
  }, {
    key: 'setDescription',
    value: function setDescription(description) {
      this.gqType.description = description;
    }
  }, {
    key: 'isFieldRequired',
    value: function isFieldRequired(fieldName) {
      return this.getFieldType(fieldName) instanceof GraphQLNonNull;
    }
  }, {
    key: 'makeFieldsRequired',
    value: function makeFieldsRequired(fieldNameOrArray) {
      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      var fields = this.getFields();
      fieldNames.forEach(function (fieldName) {
        if (fields[fieldName]) {
          if (!(fields[fieldName].type instanceof GraphQLNonNull)) {
            fields[fieldName].type = new GraphQLNonNull(fields[fieldName].type);
          }
        }
      });
      this.setFields(fields);
    }
  }, {
    key: 'makeFieldsOptional',
    value: function makeFieldsOptional(fieldNameOrArray) {
      var fieldNames = Array.isArray(fieldNameOrArray) ? fieldNameOrArray : [fieldNameOrArray];
      var fields = this.getFields();
      fieldNames.forEach(function (fieldName) {
        if (fieldNames.includes(fieldName)) {
          if (fields[fieldName].type instanceof GraphQLNonNull) {
            fields[fieldName].type = fields[fieldName].type.ofType;
          }
        }
      });
      this.setFields(fields);
    }
  }, {
    key: 'getByPath',
    value: function getByPath(path) {
      var itc = this;
      var parts = path.split('.');
      while (parts.length > 0) {
        if (!itc) return undefined;
        var name = parts[0];
        var fieldType = getNamedType(this.getFieldType(name));
        if (fieldType instanceof GraphQLInputObjectType) {
          itc = new InputTypeComposer(fieldType);
        } else {
          itc = undefined;
        }
        parts.shift();
      }
      return itc;
    }
  }]);

  return InputTypeComposer;
}();

export default InputTypeComposer;