var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import MissingType from '../type/missingType';
import compose from '../utils/compose';
import deepmerge from '../utils/deepmerge';
import { upperFirst } from '../utils/misc';
import { getProjectionFromAST } from '../projection';

import TypeComposer from '../typeComposer';

var Resolver = function () {
  function Resolver(typeComposer) {
    var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    _classCallCheck(this, Resolver);

    if (!(typeComposer instanceof TypeComposer)) {
      throw Error('First argument for Resolver.constructor should be TypeComposer instance');
    }
    this.typeComposer = typeComposer;

    if (!opts.name) {
      throw Error('For Resolver constructor the `opts.name` is required option.');
    }
    this.name = opts.name;

    this.outputType = opts.outputType || null;
    this.middlewares = opts.middlewares || [];
    this.args = opts.args || {};
    this.kind = opts.kind || null;
    this.description = opts.description || '';

    if (opts.resolve) {
      this.resolve = opts.resolve;
    }

    typeComposer.addResolver(this);
  }

  _createClass(Resolver, [{
    key: 'hasArg',
    value: function hasArg(argName) {
      return !!this.args[argName];
    }
  }, {
    key: 'getArg',
    value: function getArg(argName) {
      if (this.hasArg(argName)) {
        return this.args[argName];
      }
      return undefined;
    }
  }, {
    key: 'setArg',
    value: function setArg(argName, argConfig) {
      this.args[argName] = argConfig;
    }
  }, {
    key: 'removeArg',
    value: function removeArg(argName) {
      delete this.args[argName];
    }
  }, {
    key: 'composeArgs',
    value: function composeArgs() {
      var argsMWs = this._getMiddlewaresByKey('args').map(function (mw) {
        return mw.args;
      });

      // return compose(...argsMWs)(args => Object.assign({}, args, this.args))(this.args);
      return compose.apply(undefined, _toConsumableArray(argsMWs))(function (args) {
        return args;
      })(this.args);
    }

    /*
    * This method should be overriden via constructor
    */

  }, {
    key: 'resolve',
    value: function resolve(resolveParams) {
      // eslint-disable-line
      return Promise.resolve();
    }
  }, {
    key: 'composeResolve',
    value: function composeResolve() {
      var resolveMWs = this._getMiddlewaresByKey('resolve').map(function (mv) {
        return mv.resolve;
      });
      return compose.apply(undefined, _toConsumableArray(resolveMWs))(this.resolve);
    }
  }, {
    key: 'getResolve',
    value: function getResolve() {
      return this.resolve;
    }
  }, {
    key: 'setResolve',
    value: function setResolve(resolve) {
      this.resolve = resolve;
    }
  }, {
    key: 'getOutputType',
    value: function getOutputType() {
      if (this.outputType) {
        return this.outputType;
      }

      return MissingType;
    }
  }, {
    key: 'composeOutputType',
    value: function composeOutputType() {
      var outputTypeMWs = this._getMiddlewaresByKey('outputType').map(function (mw) {
        return mw.outputType;
      });

      return compose.apply(undefined, _toConsumableArray(outputTypeMWs))(function (outputType) {
        return outputType;
      })(this.getOutputType());
    }
  }, {
    key: 'getFieldConfig',
    value: function getFieldConfig() {
      var opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var resolve = this.composeResolve();
      return {
        type: this.composeOutputType(),
        args: this.composeArgs(),
        resolve: function (_resolve) {
          function resolve(_x3, _x4, _x5, _x6) {
            return _resolve.apply(this, arguments);
          }

          resolve.toString = function () {
            return _resolve.toString();
          };

          return resolve;
        }(function (source, args, context, info) {
          var projection = getProjectionFromAST(info);
          if (opts.projection) {
            projection = deepmerge(projection, opts.projection);
          }
          return resolve({ source: source, args: args, context: context, info: info, projection: projection });
        })
      };
    }

    /**
     * You may pass one middleware or list of them:
     *   addMiddleware(M1)
     *   addMiddleware(M1, M2, ...)
     */

  }, {
    key: 'addMiddleware',
    value: function addMiddleware() {
      var _middlewares;

      (_middlewares = this.middlewares).push.apply(_middlewares, arguments);
    }
  }, {
    key: '_getMiddlewaresByKey',
    value: function _getMiddlewaresByKey(key) {
      return this.middlewares.filter(function (mw) {
        return mw.hasMethod(key);
      });
    }
  }, {
    key: 'getKind',
    value: function getKind() {
      return this.kind;
    }
  }, {
    key: 'getDescription',
    value: function getDescription() {
      return this.description;
    }
  }, {
    key: 'clone',
    value: function clone(newTypeComposer) {
      var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var oldOpts = {};
      for (var key in this) {
        if (this.hasOwnProperty(key)) {
          // $FlowFixMe
          oldOpts[key] = this[key];
        }
      }
      return new Resolver(newTypeComposer, Object.assign({}, oldOpts, opts));
    }
  }, {
    key: 'getTypeComposer',
    value: function getTypeComposer() {
      return this.typeComposer;
    }
  }, {
    key: 'getNameCamelCase',
    value: function getNameCamelCase() {
      return upperFirst(this.name);
    }
  }, {
    key: 'wrapResolve',
    value: function wrapResolve(resolveMW) {
      var newResolver = this.clone(this.typeComposer);
      newResolver.setResolve(resolveMW(this.getResolve()));
      return newResolver;
    }
  }]);

  return Resolver;
}();

export default Resolver;